package Impedance::Header;
#$Id: Header.pm,v 1.57 2023/12/21 08:17:54 root Exp $
##########################################################
#
# Package Impedance::Header
#
# This module specifies global file locations and other 
# global variables used for the ElchemeaAnalytical software package.
#
# The module also specifies the colors of the resulting 
# gnuplot figures (this is specified in the $gpheader variable).
#
##########################################################
$ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
use strict;
use warnings;
use Fcntl qw(:DEFAULT :flock);
our $gpversion;
BEGIN {
    use Exporter ();
    our ($VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS);
    $VERSION = '1.6.2';
    $ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin';
    $gpversion = `/usr/bin/gnuplot -V`;
    if ($gpversion  && $gpversion =~ /gnuplot\s*([\d\.]+)/i)
    {
        $gpversion = $1;
    }
    else
    {
        $gpversion = 4;
    }
    %EXPORT_TAGS = (
        'standard'=>[qw/$libdir $htmldir $pngdir $homedir/]
		    );
    Exporter::export_ok_tags('standard');
    Exporter::export_tags('standard');
    @ISA = qw(Exporter);
}

my @lt = localtime();
my $year = $lt[5]+1900;

my $WEB = '/var/www/html/';
#### Global file locations ####
our $libdir = '/usr/local/bin/';
our $htmldir = $WEB.'analytic/';
our $pngdir = $htmldir.'png/';
our $homedir = '/home/analytic/';
#### End global file locations ####
our $COPYYEAR = '';
our $IMPLEMENTERS = 'S&oslash;ren Koch, Christopher Graves and Karin Vels Hansen';
our $COPYRIGHT = 'DTU Energy at Technical University of Denmark';
our $DESIGNERS = 'S&oslash;ren Koch &amp Karin Vels Hansen';
our $TESTERS = '';

### Control size of graphs ###
#my $size = 'set size 1,1'; ## Use this to resolve some issues with Gnuplot version 4.2.6     ##
#my $size = 'set size 1.3,1.1'; ## Use this for mormal operation     ##
my $size = 'set size 0.9,0.7';  ## Use this for 1024x800 operation   ##
#our $pssize = '800x600'; ## Use this for mormal operation     ##
our $pssize = '600x400'; ## Use this for 1024x800 operation   ##


our $gpheader = <<END;

set term png x000000 xffffff x000000 xffff00 x00ff00 xff9000 xff0000 xff00ff x00ffff xffffff x999999
$size
set zeroaxis linetype -1

END

our $gpheader_white = <<END;

set term png xffffff x000000 xffffff xff0000 x0000ff xff00ff x222222 x000000 x0000ff xff00ff x222222
$size
set zeroaxis linetype -1

END


if ($gpversion > 5)
{
    $gpheader = <<END;

set term pngcairo background rgb 'black'
$size
set zeroaxis linetype -1
set border lc rgb 'white'

END

    $gpheader_white = <<END;

set term pngcairo background rgb 'black'
$size
set zeroaxis linetype -1
set border lc rgb 'white'

END

}


return 1;
