#!/usr/bin/perl -T
# $Id: main.cgi,v 1.117 2016/02/03 09:25:58 sofc Exp $

use strict;
use warnings;
use Sys::Hostname;
use Impedance::Header;
use Impedance::Model;
use Impedance::IMPCGI qw(:html :cgi);
use CGI qw(:standard :netscape);
use SemaforeFile;
use File::stat;
use FileHandle;
use IPC::Open2;
my %slabels = (-3=>'mOhm',0=>'Ohm',3=>'kOhm',6=>'MOhm');
my $plotmode = param('plotmode');
$plotmode = 'Nyquist' unless ($plotmode);
my $fitmode = param('fitmode');
$fitmode = 'Inverse' unless ($fitmode);
my $admode = param('admode');
$admode = 'Impedance' unless ($admode);
print_header("Elchemea Analytical");   
top_nav_bar_start();
top_nav_bar_button_js('','load_i2b','Load impedance file',undef,'s3',"CPLE_open('load_i2b.cgi',400,300,30,30)");
top_nav_bar_button('','new_model','New simulation',undef,'s4');
top_nav_bar_button_js('','Save model','Save Model and data',undef,'s3',"report_open('print_all.cgi',600,500,30,30)");
tab_newrow;
param(-name=>'action',-value=>'') unless (param('action'));
if (param('action'))
{
    js_back();
    top_js_return();
}
else
{
    top_no_button;
}

$_ = param('action');
 CASE:
{
    ### Insert case statements ###
    /new_model/ and do {&new_model; last CASE;};
    /new_data/ and do {&new_data; last CASE;};
    ### Default response ####
    if (param('mode'))
    {
	if (param('mode') eq 'simulation')
	{
	    &model_main();
	}
	else
	{
	    &plot_main();
	}
    }
    else
    {
	&main_form;
    }
}
print <<END;

<script language="javascript">

function sofc_submit()
{
    document.sofc.submit();
}

</script>

END

&print_end;
exit;


sub model_main()
{
    param(-name=>'mode',-value=>'simulation');
    top_nav_bar_button_js('','report','report',undef,'s2',"open_report();");
    top_nav_bar_button_js('','add_element','Add element',undef,'s1',"set_fitlist();IMP_open('model.cgi',500,400,50,50)");
    top_nav_bar_button_js('','manual_edit','Manual edit model',undef,'s2',"set_fitlist();IMP_open('manual.cgi',500,400,50,50)");
    top_no_button;
    end_top_bar();
    print start_form(-name=>'show_report',-action=>'report.cgi');
    print hidden(-name=>'filename',-value=>'',-force=>1);
    print end_form();
    &print_model();
}

sub plot_main()
{
    if (param('data') && !param('model') && param('add_R_L') && param('add_R_L') =~ /Yes/i )
    {
	### Find highest frequency point for Rs and L determination ### 
	my @m = split(/\n/,param('data'));
	my %h = ();
	my $count = 0;
	my $fmaxlog = undef;
	foreach my $s (@m)
	{
	    $s =~ s/^\s*//;
	    my ($f,$r,$i) = split(/[\s\t]+/,$s);
	    next unless ($f);
	    $h{$f} = $s;
	    my $logf = log($f)/log(10);
	    $fmaxlog = $logf unless (defined($fmaxlog));
	    next if ($logf < $fmaxlog-1);
	    $count++;
	}
	my @data = ();
        foreach (reverse(sort {$a<=>$b} (keys %h)))
        {
            push @data,$h{$_}; 
        }
	my ($f,$r,$i) = split(/[\s\t]+/,$data[0]);
	$r = sprintf("%.4g",$r);
	my $l = sprintf("%.4g",$i / (2 * 3.141592 * $f));
	eval {
	    my $max = $count;
	    $max = @data if (@data < $max);
	    my $cmd = '/usr/local/bin/analytic/get_L_by_kk.py '.($max-1);
	    my $pid = open2(*Rd,*Wrt,$cmd);
	    for (my $x = 0; $x < $max; $x++)
	    {
		my $str = $data[$x];
		$str =~ s/[\s\t]+/\t/g;
		$str =~ s/[\s\t]*$//;
		print Wrt $str,"\n";
	    }
	    close Wrt;
	    $l = <Rd>;
	    chomp $l;
	    if ($l && $l =~ /\d/)
	    {
		$l = sprintf("%.4g",$l);
	    }
	};
	if (!$l || $@)
	{
	    $l = sprintf("%.4g",$i / (2 * 3.141592 * $f));
	}
	param(-name=>'model',-value=>"L: L=$l\nR: R=$r\n") unless ($admode && $admode !~ /impedance/i);
    }
    param(-name=>'mode',-value=>'plot');
    top_nav_bar_button_js('','report','report',undef,'s2',"open_report();");
    top_nav_bar_button_js('','add_element','Add element',undef,'s1',"set_fitlist();IMP_open('model.cgi',500,400,50,50)");
    top_nav_bar_button_js('','manual_edit','Manual edit model',undef,'s2',"set_fitlist();IMP_open('manual.cgi',500,400,50,50)");
    top_nav_bar_button_js('','fit','Fit model to data',undef,'s0','start_fit();');
    end_top_bar();
    print start_form(-name=>'show_report',-action=>'report.cgi');
    print hidden(-name=>'filename',-value=>'',-force=>1);
    print end_form();
    &print_model();
}

sub new_data
{
    param(-name=>'data',-value=>''); ### Unset old data ###
    param(-name=>'mode',-value=>'plot');
    top_no_button;
    top_nav_bar_button_js('','add_element','Add element',undef,'s2',"IMP_open('model.cgi',500,400,50,50)");
    top_no_button;
    top_no_button;
    end_top_bar();
    &print_data_new();
}

sub new_model
{
    param(-name=>'model',-value=>''); ### Unset old model ###
    param(-name=>'data',-value=>''); ### Unset old data ###
    param(-name=>'mode',-value=>'simulation');
    top_no_button;
    top_nav_bar_button_js('','add_element','Add element',undef,'s2',"IMP_open('model.cgi',500,400,50,50)");
    top_no_button;
    top_no_button;
    end_top_bar();
    &print_model();
}


sub print_data_new()
{
    print '<center>'; 
    print h1('Enter new data to plot / fit');
    print start_form(-name=>'new_data');
    print '<table><tr><td colspan=2 width="400">';
    print textarea(-name=>'data',-value=>'',-force=>1,-class=>'textfield',-rows=>30);
    unless (param('model'))
    {
	print '</td></tr><tr><th width=300> Add series R and L : </th><td style="text-align: right;">';
	print popup_menu(-name=>'add_R_L',-values=>['No','Yes']);
    }
    print '</tr><td colspan=2>';
    print button(-name=>'Load',-value=>'Load',-class=>'s3',-onclick=>'set_data()');
    print '</td></tr></table>';
    print end_form;
    print '</center>';
    print <<END;

<script langhage="javascript">

function set_data()
{
    document.sofc.data.value = document.new_data.data.value;
    document.sofc.mode.value = 'plot';
    document.sofc.submit();
}

</script>

END

}


sub print_model($)
{
    my $mode = param('mode');
    my $minf = get_CGI_value_clean('minf');
    my $maxf = get_CGI_value_clean('maxf');
    my $bgcolor = get_CGI_value_clean('bgcolor');
    my $arcs = get_CGI_value_clean('arcs');
    $maxf = 1e100 if ($maxf > 1e100);
    $minf = 1e-100 if ($minf < 1e-100);
    my $model = Impedance::Model->new();
    my $m = Impedance::IMPCGI::format_model(param('model'));
    my $scale = get_CGI_value_clean('scale_factor');
    $scale = 0 unless ($scale);
    my $scalefn = 10**(-$scale);
    $model->parse($m);
    $model->data(param('data'));
    $model->set_show_arcs($arcs);
    $model->scale_factor($scalefn);
    $model->set_xlabel("Z_real(".$slabels{$scale}.")");
    $model->set_ylabel("-Z_imag(".$slabels{$scale}.")");
    $model->set_ylabel_bode("Z(".$slabels{$scale}.")");
    $model->set_admittance_mode($admode) if ($admode);
    print '<center>';
    print h3({-id=>'Header'},$model->model_text);
    print '<table border=0><tr><td>';
    print start_form(-name=>'fitmodel');
    print_hidden();
    action('fit');
    print '<table border=0 cellpadding=10><tr>';
    if ($mode eq 'simulation')
    {
	print  '<tr><td colspan=3><hr></td></tr>';
	print '<th colspan=2>Minimum frequency</th><td>';
	print textfield(-name=>'min',-value=>$minf,-force=>1,-onchange=>'document.sofc.minf.value=this.value;document.sofc.submit();',-size=>8);
	print '</td></tr>';
	print '<tr><th colspan=2>Maximum frequency</th><td>';
	print textfield(-name=>'max',-value=>$maxf,-force=>1,-onchange=>'document.sofc.maxf.value=this.value;document.sofc.submit();',-size=>8);
	print '</td></tr>';
    }
    print  '<tr><td colspan=3><hr></td></tr>' if (param('model'));
    my @flist = ();
    foreach my $e ($model->elements)
    {
	print '<tr>';
	push @flist,(Impedance::IMPCGI::p_element($model->element($e)));
	print '</tr>';
	print  '<tr><td colspan=3><hr></td></tr>' if (param('model'));
    }
    print '<tr><th colspan=3 id="topf"></th></tr>';
    print '</table>';
    print end_form();
    print '</td><td>';
    my @adv_opt = ();
    my $xmin = get_CGI_value_clean('xmin');
    my $xmax = get_CGI_value_clean('xmax');
    my $ymin = get_CGI_value_clean('ymin');
    my $ymax = get_CGI_value_clean('ymax');
    foreach ($xmin,$ymin,$xmax,$ymax)
    {
	$_ = '' unless defined($_);
    }
    foreach my $tag ('xmin','xmax','ymin','ymax')
    {
	my $val = get_CGI_value_clean($tag);
	$model->set_limit($tag,$val) if (defined($val) && $val =~ /\d/);
    }
    push @adv_opt, "set xrange [$xmin:$xmax]";
    push @adv_opt, "set yrange [$ymin:$ymax]";
    push @adv_opt, param('gp_text');
    my $file = SemaforeFile->new('/tmp/model_'.$$.'.gp');
    my $file2 = SemaforeFile->new('/tmp/bode_'.$$.'.gp');
    my $pngfn = $pngdir.'model_'.$$.'.png';
    my $pngb = $pngdir.'bode_'.$$.'.png';
    if ($bgcolor && $bgcolor =~ /white/i)
    {
	$file->writeline($Impedance::Header::gpheader_white);
	$file2->writeline($Impedance::Header::gpheader_white);	
    }
    else
    {
	$file->writeline($Impedance::Header::gpheader);
	$file2->writeline($Impedance::Header::gpheader);
    }
    $file->append("set out '$pngfn'");
    $file2->append("set out '$pngb'");
    my $title = param('filename');
    if ($title)
    {
	$title .= ', '.$model->model_text;
    }
    else
    {
	$title = $model->model_text;
    }
    $model->adv_opt(@adv_opt);
    $file->append("set title '$title'");
    $file2->append("set title '$title'");
    eval {
	if ($mode eq 'plot')
	{
	    $file->append($model->print_plot);
	    $file2->append($model->print_bode);
	}
	else
	{
	    $file->append($model->print_imp_sim($minf,$maxf));
	    $file2->append($model->print_bode_sim($minf,$maxf));
	}
	system('/usr/bin/gnuplot '.$file->filename);
	system('/usr/bin/gnuplot '.$file2->filename);
    };
    if ($@)
    {
	print h3({-class=>'Alert'},$@);
    }
    unlink $file->filename if ($file->exist);
    unlink $file2->filename if ($file2->exist);
    my $imp_pub_png = '/analytic/png/model_'.$$.'.png';
    my $bode_pub_png = '/analytic/png/bode_'.$$.'.png';
    #### Hook for Elchemea or RFCcontrol####
    if ((-e '/home/elchemea/' && param('E_file')) || (-e '/home/celltest/' && param('RFC_file')))
    {
	my $efile = param('E_file');
	$efile = param('RFC_file') unless ($efile);
	if ($efile =~ /^([\w\/\-\:]+)\/([\w\.\-\:]+)$/)
	{
	    my $dir = $1;
	    my $f = $2;
	    opendir MYDIR,$dir or die "Could not open directory $dir $!";
	    my @flist = ();
	    if (param('E_file'))
	    {
		@flist = sort {$a cmp $b} grep /\w+_\d+\.i2b/, readdir(MYDIR);
	    }
	    else
	    {
		@flist = sort {$a cmp $b} grep /[\w\-\:]+.i2b/, readdir(MYDIR);
	    }
	    my $id = 0;
	    while ($flist[$id] ne $f && $id < @flist) { $id++ };
	    closedir(MYDIR);
	    print '<table border=0 cellpadding=10 align=center><tr><td width=200>';
	    if ($id > 0)
	    {
		my @data = ();
		my $prev = $dir.'/'.$flist[$id-1];
		my $efn = SemaforeFile->new($prev);
		my @lines = $efn->readlines();
		my $name;
		my $x = 0;
		while ($x++ < @lines)
		{
		    next unless (defined($lines[$x]) && $lines[$x] =~ /^[\s\t]*\d+[\s\t]*$/);
		    last;
		}
		while ($x++ < @lines)
		{
		    push @data,$lines[$x] if (defined($lines[$x]) && $lines[$x] =~ /\d/ && $lines[$x] =~ /[123456789]/);
		}
		$name = $flist[$id-1];
		my $data = join('\\n',@data);
		if (param('E_file'))
		{
		    print button(-name=>'prev',-value=>'Previous file',-class=>'s5',-onclick=>"document.sofc.data.value='$data';document.sofc.filename.value='$name';document.sofc.E_file.value='$prev';set_fitlist();document.sofc.submit()");
		}
		else
		{
		    print button(-name=>'prev',-value=>'Previous file',-class=>'s5',-onclick=>"document.sofc.data.value='$data';document.sofc.filename.value='$name';document.sofc.RFC_file.value='$prev';set_fitlist();document.sofc.submit()");
		}    
	    }
	    print '</td><td width=200>';
	    print '</td><td width=200>';
	    if ($id < @flist-1)
	    {
		my @data = ();
		my $prev = $dir.'/'.$flist[$id+1];
		my $efn = SemaforeFile->new($prev);
		my @lines = $efn->readlines();
		my $name;
		my $x = 0;
		while ($x++ < @lines)
		{
		    next unless (defined($lines[$x]) && $lines[$x] =~ /^[\s\t]*\d+[\s\t]*$/);
		    last;
		}
		while ($x++ < @lines)
		{
		    push @data,$lines[$x] if (defined($lines[$x]) && $lines[$x] =~ /\d/ && $lines[$x] =~ /[123456789]/);
		}
		$name = $flist[$id+1];
		my $data = join('\\n',@data);
		if (param('E_file'))
		{
		    print button(-name=>'next',-value=>'Next file',-class=>'s5',-onclick=>"document.sofc.data.value='$data';document.sofc.filename.value='$name';document.sofc.E_file.value='$prev';set_fitlist();document.sofc.submit()");
		}
		else
		{
		    print button(-name=>'next',-value=>'Next file',-class=>'s5',-onclick=>"document.sofc.data.value='$data';document.sofc.filename.value='$name';document.sofc.RFC_file.value='$prev';set_fitlist();document.sofc.submit()");
		}
	    }
	    print '</td></tr></table><br>';
	}
    }
    if ($plotmode && $plotmode =~ /bode/i)
    {
	if ($mode eq 'simulation')
	{
	    print '<img name="simulation" align=center src="'.$bode_pub_png.'" id="simulation">';
	}
	else
	{
	    print "<img name='simulation' align=center src='".$bode_pub_png."' id='simulation' onclick='delete_point_form()'>";
	}
    }
    else
    {
	if ($mode eq 'simulation')
	{
	    print '<img name="simulation" align=center src="'.$imp_pub_png.'" id="simulation">';
	}
	else
	{
	    print "<img name='simulation' align=center src='".$imp_pub_png."' id='simulation' onclick='delete_point_form()'>";
	}
    }
    
    print '<br>';
    print '<table border=0 cellpadding=10 align=center><tr><td>';
    print popup_menu(-name=>'file_type',-id=>'file_type',-style=>'visibility:hidden;',-values=>['report_bode','report_model','report_error'],-default=>param('mode'),-force=>1,-labels=>{'report_bode' => 'Bode','report_model' => 'Nyquist','report_error' =>'Error'});
    print '</td><td>';
    print button(-id=>'download_file',-style=>'visibility:hidden;',-name=>'dummy',-value=>'Download eps file',-class=>'s8',-OnClick=>'download_file()');
    print '</td><td width=50>';
    print '</td><td>';
    print button(-name=>'fit_tab',-value=>'Show fit table',-class=>'s9',-OnClick=>'print_fit_table(document.sofc.fit_table.value)');
    print '</td></tr></table>';

    print '</center></td><td>Plot type<br>';
    print radio_group(-name=>'mode',-id=>'mode',-values=>['Nyquist','Bode'],-default=>$plotmode,-force=>1,-onchange=>'set_mode(this)');
    print '<br><br><br><br>Fit weight<br>';
    print popup_menu(-name=>'fitmode',-id=>'fitmode',-values=>['Even','Inverse'],-labels=>{'Even' => 'Even weight','Inverse' => 'Inverse modulus'},-default=>$fitmode,-force=>1,-onchange=>'set_fitmode(this)',-class=>'textfield');
    print '<br><br><br><table border=0><tr><th width="55%">Background:</th><th width="45%">';
    print popup_menu(-name=>'bgcolor',-id=>'bgcolor',-values=>['Black','White'],-default=>$bgcolor,-force=>1,-onchange=>'set_color(this)',-class=>'textfield');
    print '</tr><tr><th>Show arcs:</th><th>';
    print popup_menu(-name=>'arcs',-id=>'arcs',-values=>[0,1],-default=>$arcs,-labels=>{'0' =>'No','1' => 'Yes'},-force=>1,-onchange=>'set_arc(this)',-class=>'textfield');
    print '</tr><tr><th colspan=2>';
    print button(-name=>'dummy2',-value=>'Advanced plot options',-onclick=>'edit_advanced()',-class=>'s5');
    print '</th></table></td></tr>';
    print '</table>';
    print '</center>';
    print start_form(-name=>'fit_report');
    print hidden(-name=>'fit_result');
    print hidden(-name=>'filename');
    print hidden(-name=>'imp_plot');
    print hidden(-name=>'error_plot');
    print hidden(-name=>'final_pars');
    print hidden(-name=>'info');
    print hidden(-name=>'impfile',-value=>$imp_pub_png,-force=>'1');
    print hidden(-name=>'bodefile',-value=>$bode_pub_png,-force=>'1');
    print hidden(-name=>'model_name',-value=>$model->model_text,-force=>1);
    print end_form();
    print start_form(-name=>'download');
    print end_form();
    &print_model_js_manipulate(@flist);
    my $dir = $Impedance::Header::htmldir.'/png/';
    opendir DIR,$dir or die "Could not open directory '$dir' $!\n";
    while (my $file = readdir(DIR))
    {
	next unless ($file =~ /png/);
	my $fn = $dir.$file;
	my $sb = stat($fn);
	next unless ((time-300) > $sb->mtime);
	if ($fn =~ /([\w\.\/]+)/)
	{
	    unlink $1 if -e ($1);
	}
    }
    closedir(DIR);
}


sub print_model_js_manipulate
{
    my @fitlist = @_;
    my $str = '';
    for (my $x = 0; $x < @fitlist; $x++)
    {
	$str .= 'fitarr['.$x."] = '".$fitlist[$x]."';\n";
    }

    print <<END;

<script language=javascript>

var fileid = 0;
var fitarr = new Array;
$str

var header = '';
var report_pdf = '';

function download_file()
{
    var e1 = document.getElementById('file_type');
    var filename = "/analytic/png/"+e1.value+'_'+fileid+'.eps';
    document.download.action = filename;
    document.download.submit();
}

function edit_advanced()
{
    var w = window.open('advanced_options.cgi', 'adv', 'width=500,height=300,resizeable,scrollbars');
    w.focus();
}


function set_download_file(display)
{
    var e1 = document.getElementById('file_type');
    var e2 = document.getElementById('download_file');
    if (display)
    {
	if (e1 != null)
	    e1.style.visibility="visible";
	if (e2 != null)
	    e2.style.visibility="visible";	
    }
    else
    {
	if (e1 != null)
	    e1.style.visibility="hidden";
	if (e2 != null)
	    e2.style.visibility="hidden";
    }
}

function print_fit_table(res)
{
    res = res.replace(/\\n+/g,"<br>");
    var w = window.open('', '', 'width=750,height=550,resizeable,scrollbars');
    w.document.write(res);
    w.document.close();
}


function delete_point_form()
{
    var script = 'delete_point.cgi';
    var data = document.sofc.data.value;
    var model = document.sofc.model.value;
    var t = document.sofc.type.value;
    var hidden = "?data="+data+"&model="+model+"&type="+t+"&plotmode="+document.sofc.plotmode.value;
    hidden = hidden.replace(/\\r/g,"\\n");
    hidden = hidden.replace(/\\n/g,"%0A");
    hidden = hidden.replace(/\\+/g,"%2B");
    var myWindow=window.open(script+hidden,'new_win_delete_point','width=900,height=750,left=100,top=100');
    myWindow.location.href = script+hidden;
    if (!myWindow.opener)
        myWindow.opener = self;
    myWindow.focus();
}

function set_mode(i)
{
    document.sofc.plotmode.value = i.value;
    var filename = document.fit_report.impfile.value;
    if (i.value == 'Bode')
    {
	filename = document.fit_report.bodefile.value;
    }
    var img = document.getElementById("simulation");
    img.src = filename + "?" + Math.random();
}

function set_fitmode(i)
{
    document.sofc.fitmode.value = i.value;
}

function set_color(i)
{
    document.sofc.bgcolor.value = i.value;
    document.sofc.submit();
}

function set_arc(i)
{
    document.sofc.arcs.value = i.value;
    document.sofc.submit();
}

function open_report()
{
    if (report_pdf.length < 5)
    {
	alert('No report avaliable yet');
	return false;
    }
    document.show_report.filename.value = report_pdf;
    document.show_report.submit();
}

function start_fit()
{
    report_pdf = '';
    var element = document.getElementById("Header");
    header = element.innerHTML;
    element.innerHTML = "Working . . . .";
    set_fitlist();
    var fl = document.sofc.fitlist.value;
    var arr = new Array;
    var inp = fl.split(",");
    for (x = 0; x < inp.length; x++)
    {
	var a = inp[x].split("fit_");
	var item = a[1].split("=");
	if (item[1] == "1")
	    arr.push(item[0]);
    }
    var fit = arr.join(",");
    var m = document.sofc.model.value;
    var d = document.sofc.data.value;
    var f = document.sofc.filename.value;
    var fmode = document.sofc.fitmode.value;
    var bgc = document.sofc.bgcolor.value;
    var arc = document.sofc.arcs.value;
    var xmin = document.sofc.xmin.value;
    var xmax = document.sofc.xmax.value;
    var ymin = document.sofc.ymin.value;
    var ymax = document.sofc.ymax.value;
    var scale = document.sofc.scale_factor.value;
    var admode = document.sofc.admode.value;
    var limit = '&xmin='+xmin+'&xmax='+xmax+'&ymin='+ymin+'&ymax='+ymax+'&scale_factor='+scale+"&admode="+admode;
    var q = "&action=fit_pdf&fitlist="+fit+"&model="+m+"&data="+d+"&filename="+f+"&fitmode="+fmode+"&bgcolor="+bgc+"&arcs="+arc+limit;
    q = q.replace(/\\n/g,"%0A");
    q = q.replace(/\\+/g,"%2B");
    run_ajax('ajax_model.cgi',q,fit_result);
}

function fit_result(res)
{
    set_download_file(true);    
    document.sofc.fit_res.value = res;
    var element = document.getElementById("Header");
    element.innerHTML = header;
    var values = new Array;
    var maxf = new Array;
    var text = new Array;
    var fiterror = new Array;
    var out = res.split("\\n");
    if (out[0] != "OK")
    {
	alert(res);
	return false;
    }
    var x = 0;
    var re = /INFO:/;
    var re2 = /RES:/;
    var reps = /Pseudo/;
    var endre = /FITDATA/;
    var fiterr = /FITERROR/;
    while (++x < out.length)
    {
	if (endre.test(out[x]))
	    break;
	if (reps.test(out[x]))
	{
	    var item = out[x].split(": ");
	    var val = parseFloat(item[2]);
	    val = val.toPrecision(4);
	    maxf.push(item[1] + " " + val + " F");
	}
	else if (re.test(out[x]))
	{
	    var item = out[x].split(": ");
	    var val = parseFloat(item[2]);
	    val = val.toPrecision(4);
	    maxf.push(item[1] + " " + val + " Hz");
	}
	else if (re2.test(out[x]))
	{
	    var item = out[x].split(": ");
	    item.shift();
	    maxf.push(item.join(': '));
	}
	else if (fiterr.test(out[x]))
	{
	    var item = out[x].split(": ");
	    item.shift();
	    fiterror.push(item.join(': '));
	}
	else
	{
	    values.push(out[x]);
	}
    }
    x++;
    x++;
    x++;
    var re = /END/;
    while (++x < out.length)
    {
	if (re.test(out[x]))
	    break;
	text.push(out[x]);
    }
    var file = out[++x].split(":");
    var filename = file[1];
    file = out[++x].split(":");
    document.fit_report.imp_plot.value = file[1];
    file = out[++x].split(":");
    document.fit_report.error_plot.value = file[1];
    var errfile = file[1];
    var errpop = out[++x];
    report_pdf = out[++x];
    var img = document.getElementById("simulation");
    document.fit_report.impfile.value = '/analytic/png/'+filename;
    var bodefile = filename.replace('model','bode');
    document.fit_report.bodefile.value = '/analytic/png/'+bodefile;
    if (document.sofc.plotmode.value == 'Bode')
    {
	filename = bodefile;
    }
    img.src = '/analytic/png/' + filename + "?" + Math.random();
    document.fit_report.fit_result.value = "XXXXX    FIT RESULT   XXXXX\\n\\n"+text.join("\\n");
    document.fit_report.final_pars.value = values.join("\\n");
    document.fit_report.info.value = maxf.join("\\n");
    if (confirm("Fit resulted in the following parameters:\\n\\n"+document.fit_report.final_pars.value+"\\n\\nCopy values to start values"))
    {
	var fr = values.join("\\n");
	set_start_values_fit(fr);
	var fit_tab_line = document.sofc.filename.value + "\\t" + values.join("\\t") + "\\t" + maxf.join("\\t") + "\\t" + fiterror.join("\\t");
	if (document.sofc.fit_table.value.length > 10)
	{
	    document.sofc.fit_table.value = document.sofc.fit_table.value + "\\n" + fit_tab_line;
	}
	else
	{
	    document.sofc.fit_table.value = fit_tab_line;
	}
    }
    fileid = 0;
    var erf = errpop.split(":");
    try {
	var rx = /\\_(\\d+)\\./;
	var rxval = rx.exec(erf[1]);
	fileid = rxval[1];
	var mywin = window.open("/analytic/png/"+erf[1],"errorplot",'width=700,height=350,left=700,top=100');
	mywin.location.href = "/analytic/png/"+erf[1];
	if (!mywin.opener)
	{
	    mywin.opener = self;
	}
	mywin.focus();
    }
    catch(error)
    {
	alert(error);
    }
    var element = document.getElementById("topf");
    element.innerHTML = maxf.join("<br>");
}

function set_start_values_fit(res)
{
    var arr = res.split("\\n");
    var modarr = document.sofc.model.value.split("\\n");
    for (x = 0; x < arr.length; x++)
    {
	var item = arr[x].split(":");
	var tag = 'value_'+item[0];
	var element = document.getElementById(tag);
	element.value = item[1];
	var mitem = item[0].split("_");
	var nr = mitem[1]-1;
	var t = mitem[0];
	var line = modarr[nr];
	var larr = line.split(": ");
	if (larr.length > 1)
	{
	    var cpl = false;
	    if (larr[1].indexOf("(") > 0)
		cpl = true;
	    var tags = larr[1].split(", ");
	    for (y = 0; y < tags.length;y++)
	    {
		var vitem = tags[y].split("=");
		if (vitem[0] != t)
		    continue;
		vitem[1] = item[1];
		tags[y] = vitem.join("=");
		break;
	    }
	    larr[1] = tags.join(", ");
	    if (cpl)
		larr[1] = larr[1] + ' (';
	    modarr[nr] = larr.join(": ");
	}
    }
    document.sofc.model.value = modarr.join("\\n");
}




function set_fitlist()
{
    var fitl = '';
    for (x = 0; x < fitarr.length; x++)
    {
	if (fitl != "")
	    fitl += ","
	var e = document.getElementById(fitarr[x]);
	if (e.checked == true)
	{
	    fitl += e.name + "=1"
	}
	else
	{
	    fitl += e.name + "=0"
	}
    }
    document.sofc.fitlist.value = fitl;
    set_download_file(false);
}

function start_marker()
{
    return "";
}

function end_marker()
{
    return "";
}


function add_element_go()
{
    set_fitlist();
    document.sofc.submit();
}

function delete_element(e)
{
    if (!confirm("Delete element "+e.name))
	return false;
    document.sofc.add_R_L.value = 'No';
    var arr = e.name.split("_");
    var cpl = false;
    if (arr[0].indexOf("(") > 0)
	cpl = true;
    mod = document.sofc.model.value;
    var modarr = mod.split("\\n");
    var res = new Array;
    for (x = 0; x < modarr.length;x++)
    {
	if (x != arr[1]-1)
	{
	    res.push(modarr[x]);
	}
	else
	{
	    if (cpl)
		res.push('(');
	}	
    }
    document.sofc.model.value = res.join("\\n");
    set_fitlist();
    document.sofc.submit();
}

function set_new_val(e)
{
    set_new_val_nosubmit(e);
    set_fitlist();
    document.sofc.submit();
}

function set_new_val_nosubmit(e)
{
    var arr = e.name.split("-");
    var val = e.value;
    mod = document.sofc.model.value;
    var modarr = mod.split("\\n");
    var line = modarr[arr[0]-1];
    var cpl = false;
    if (line.indexOf("(") > 0)
    {
	cpl = true;
    }
    var linedef = line.split(": ");
    var elements = linedef[1].split(", ");
    for (x = 0; x < elements.length;x++)
    {
	var item = elements[x].split("=");
	if (item[0] == arr[1])
	{
	    elements[x] = item[0]+'='+val;
	}
    }
    linedef[1] = elements.join(", ");
    modarr[arr[0]-1] = linedef.join(": ");
    if (cpl)
    {
	modarr[arr[0]-1] = modarr[arr[0]-1] + ' (';
    }
    document.sofc.model.value = modarr.join("\\n");
}

function add_complex(t)
{
    var model = '';
    var nr = 0;
    var length = 0;
    document.sofc.offset.value = nr;
    document.sofc.length.value = length;
    document.sofc.text.value = model;
    document.sofc.type.value = t;
    set_fitlist();
    CPLE_open('complex_add.cgi',700,600,500,50);
}

function edit_complex(e)
{
    var model = document.getElementById('model_'+e.name);
    var nr = document.getElementById('nr_'+e.name);
    var length = document.getElementById('length_'+e.name);
    document.sofc.offset.value = nr.value;
    document.sofc.length.value = length.value;
    document.sofc.text.value = model.value;
    document.sofc.type.value = '';
    set_fitlist();
    CPLE_open('complex_add.cgi',700,600,50,50);
}

</script>

END

}

sub main_form
{
    top_nav_bar_button_js('','load','Load saved model',undef,'s4',"CPLE_open('load.cgi',400,300,30,30)");
    top_nav_bar_button('','new_data','Load new data',undef,'s3');
    top_nav_bar_button('about.cgi','test','About Elchemea Analytic',undef,'s3');
    top_nav_bar_button('multiplot.cgi','test','Plot multiple files',undef,'s6');
    end_top_bar();
    print '<center>';
    my $server = hostname;
    print h1();
    print '<img name="splash" src="/analytic/elchemeaanalytical.png" id="splash">';
    print '</center>';
}
